%% Code Description: Main File Table 1

% NOTE: This file is adapted from Kitamura and Stoye (2017). I am very
% grateful to them for the high quality of the files. 

% PARAMETER INSTRUCTIONS
%   (1) budget_length is the length of sub-budgets.  For example, if
%   budget_length=8, then we compute test statistics for the time periods
%   1975-1982, 1976-1983, ect.
%
%   (2) number_classes is the number of aggregate commodity bundles.
%   For example, number_classes = 3 corresponds to the commodity bundles
%   {food, services, durables}. 
%   Currently number_classes in {3,4,5} are valid inputs.  Alter
%   RUM_21_budgets.m to change aggregation methods.
%   
%   (4) polynomial_degree is the polynomial degree used in the series 
%   estimation.  
%
%   (5) estimator in {0,1,2} computes the test statistic based on 
%   kernel estimator (estimator=0), series estimator (estimator = 1)
%   or endogenous estimator (estimator =2).
%   
%   (6) genAX indicators to compute matrix of pseudo-agents  A,X and comp
%   If A,X not saved (Input folder), then program computes A,X.  
%   WARNING: If one changes the price sequence or budget set, then do not
%   use an old version of A,X, as they will be incorrect.
%  
%   (7) indJ indicator to compute the test statistic.
%
%   In practice, we should always set genAX=1 and indJ=1 ... this is mainly
%   used as a debugging device, so that we do not have to recompute AX when
%   checking to see if the test statistic works.
%
%   (8) cores pre-specifies the number of cores to use.  This is to prevent
%   overuse of server resources.
%
%
%   Other parameters:
%   (a) start_year, end_year are both determined by the dataset.  E.g. the
%   current dataset starts at 1975 and ends in 1999.
%


%% Clear Memory and start timer
clear
clc 

warning('off','MATLAB:nargchk:deprecated')

%% User-specified parameters
budget_length    = 7;       % Number of budgets
number_classes   = 3;       % Number of aggregate commodity bundles
polynomial_degree= 3;       % Series degree estimator
estimator        = 0;       % 0 = Kernel; 1 = Exogenous Series; 2 = Endogenous Series
genAX            = 1;       % Generate pseudo-agent matrix A and patches X


%% Global parameters
global classes budget_l budget_n start_year end_year periods 
global flag_genAX 


%% Flags
% flag_genAX = 1 iff we want to generate matrix A and X and there does not
% exist a pre-generated AX file.
flag_genAX = genAX;
nameAX= strcat('Input/RUM_',num2str(budget_length),'LengthBudget_',num2str(number_classes),'.mat');
if exist(nameAX) ~= 2 & genAX == 0
    flag_genAX = 1;
end

%% Parameters
% Length of sub-budgets to be called (e.g. budget_l = 8 implies budgets 
% span 75-82, 76-83, ect.).  When there are 3 classes use budget_l <9,
% otherwise it takes a long time to run.
budget_l    = budget_length;     

% The number of commodity bundles is called classes e.g. 3 is food, 
% services, durables).  Model currently allows for classes = 3, 4, or 5.
% This can be further disaggregated by modifying RUM_21_budgets.m
classes     = number_classes;      

% Time parameters:
start_year  = 75;                        % First year for budgets
end_year    = 99;                        % Last year for budgets
periods     = end_year - start_year + 1; % Total number of periods
budget_n    = periods - (budget_l-1);    % Number of sub-budgets


%% Create budgets 
% RUM_21_budgets cleans the dataset and outputs normalized budgets, as well
% as expenditure shares and income for each surveyed household. 
[budgets_all,shares_all,income_all,instrument_all,medexp_all] = RUM_21_budgets_abi;


%% Demand predictions at each budget unconstrained 
[phat, qhat, income_long, shares_long, fw_long, period_long, x0_long, ...
            p0_long, Nyear_long] = MC_unconstrained(budgets_all,shares_all,income_all, ...
                            medexp_all, budget_n, budget_l);
garp_results = NaN(budget_n,1);
for i = 1 : budget_n
    garp_results(i) = garp(phat(:,:,i), qhat(:,:,i));
end

%% Demand predictions constrained to be mutually consistent 
results = NaN(budget_n,7);
weights_ = cell(1, budget_n);
all_q = NaN(number_classes, budget_l, budget_n);
for i = 1 : budget_n 
    fw = fw_long{i};
    tic
    [A, b, C, d, ctype, Aeq, beq, lb, ub] = mutcon_constraints_extrans(fw, ...
                                                phat(:,:,i), x0_long(1,:,i), ...
                                                Nyear_long{i});
    [xhat, resnorm] = ...
      cplexlsqmilp (C, d, A, b, ...
                    Aeq, beq, [ ], [ ], [ ], lb, [], ctype, [ ], []);
    time = toc;
    N = size(fw, 1);
    qhat_ = NaN(number_classes, budget_l);
    for t = 1 : budget_l
        qhat_(1,t) = sum(fw(:, t, 1).*xhat(1:N));
        qhat_(2,t) = sum(fw(:, t, 2).*xhat(1:N));
    end
    weights_{1,i} = xhat(1:N);
    qhat_(3,:) = x0_long(1,:,i) - sum(phat(1:2,:,i).*qhat_(1:2,:));
    all_q(:,:,i) = qhat_;
    results(i,1:5) = [size(A, 1) size(A, 2) time resnorm N];
    results(i,6) = garp(phat(:,:,i), qhat(:,:,i));
    results(i,7) = garp(phat(:,:,i), qhat_);
end
save('MC_results_explain', 'results', 'weights_', 'all_q', 'qhat', 'phat')

%% Draw graphs
load('MC_results_explain')
all_q = NaN(number_classes, budget_l, budget_n);
for i = 1 : budget_n 
    fw = fw_long{i};
    N = size(fw, 1);
    qhat_ = NaN(number_classes, budget_l);
    xhat = weights_{1,i};
    for t = 1 : budget_l
        qhat_(1,t) = sum(fw(:, t, 1).*xhat);
        qhat_(2,t) = sum(fw(:, t, 2).*xhat);
    end
    weights_{1,i} = xhat(1:N);
    qhat_(3,:) = x0_long(1,:,i) - sum(phat(1:2,:,i).*qhat_(1:2,:));
    all_q(:,:,i) = qhat_;
end

% distribution of the weight variables
for i = 1:budget_n
    w_ = weights_{i};
    figure;
    histogram(w_, 'BinWidth', .01, 'BinLimits', [.75, 1.25])
    axis([.75 1.25 0 1200])
    xlabel('Weight')
    ylabel('Frequency')
    saveas(gcf, sprintf('weight_hist_%d.png', i));
end 

% plot the budget

for i = 1:budget_l
    figure
    if i ==1 
        col = [1 0 0];
    elseif i==2
        col = [0 0 1];
    elseif i==3
        col = [0 1 0];
    elseif i==4
        col = [1 1 0];
    elseif i==5
        col = [0 1 1];
    elseif i==6
        col = [1 0 1];
    elseif i==7
        col = [.85,.325,.098];
    end
    x_ = sum(phat(:,i,1).*qhat(:,i,1));
    ax_ = x_./phat(:,i,1);
    points = diag(ax_)';
    scatter3(qhat(1,i,1), qhat(2,i,1), qhat(3,i,1), 25, col, 'filled')   
    hold on    
    scatter3(all_q(1,i,1), all_q(2,i,1), all_q(3,i,1), 25, [ 0 0 0], 'filled')
    fill3(points(1,:),points(2,:), points(3,:),col, 'FaceAlpha', .1)
%     alpha(0.1)
    xlabel('q^1')
    ylabel('q^2')
    ylabel('q^3')
    view([46.5 18.8])
    axis([5 16 5 16 5 16])
    grid on
    legend('Unconstrained', 'Constrained', 'Location', 'northoutside', 'Orientation', 'horizontal')
    saveas(gcf, sprintf('budget1_%d.png', i));
end

%% Compute X and A
% budgets{i} is the ith sub-budget.
budgets{budget_n} = [];
for ii = 1:budget_n
    tmp = phat(:,:,ii)';
    tmp = tmp./repmat(x0_long(1,:,ii)', 1, number_classes);
    budgets{ii} = tmp;
end


% We compute X and A for each budget{ii} 
time_taken_AX = NaN(1, budget_n);
if flag_genAX == 0
    load(nameAX)
else
    X{budget_n} = [];
    A{budget_n} = [];
    fprintf('Starting to generate X and A using %d-length budgets \n',budget_l)
    for ii = 1:budget_n
        tic
        X{ii} = RUM_31_genX(budgets{ii});
        A{ii} = RUM_41_genA(X{ii});
        time_taken_AX(ii) = toc;
        fprintf('Completed budget %d out of %d. \n',ii,budget_n);
    end
end

n_types = zeros(1, budget_n);
for ii = 1 : budget_n
    n_types(ii) = size(A{ii}, 2);
end

