%% Code Description: Main File for Table 2

% NOTE: This file is adapted from Kitamura and Stoye (2017). I am very
% grateful to them for the high quality of the files. 

% PARAMETER INSTRUCTIONS
%   (1) budget_length is the length of sub-budgets.  For example, if
%   budget_length=8, then we compute test statistics for the time periods
%   1975-1982, 1976-1983, ect.
%
%   (2) number_classes is the number of aggregate commodity bundles.
%   For example, number_classes = 3 corresponds to the commodity bundles
%   {food, services, durables}. 
%   Currently number_classes in {3,4,5} are valid inputs.  Alter
%   RUM_21_budgets.m to change aggregation methods.
%   
%   (4) polynomial_degree is the polynomial degree used in the series 
%   estimation.  
%
%   (5) estimator in {0,1,2} computes the test statistic based on 
%   kernel estimator (estimator=0), series estimator (estimator = 1)
%   or endogenous estimator (estimator =2).
%   
%   (6) genAX indicators to compute matrix of pseudo-agents  A,X and comp
%   If A,X not saved (Input folder), then program computes A,X.  
%   WARNING: If one changes the price sequence or budget set, then do not
%   use an old version of A,X, as they will be incorrect.
%  
%   (7) indJ indicator to compute the test statistic.
%
%   In practice, we should always set genAX=1 and indJ=1 ... this is mainly
%   used as a debugging device, so that we do not have to recompute AX when
%   checking to see if the test statistic works.
%
%   (8) cores pre-specifies the number of cores to use.  This is to prevent
%   overuse of server resources.
%
%
%   Other parameters:
%   (a) start_year, end_year are both determined by the dataset.  E.g. the
%   current dataset starts at 1975 and ends in 1999.
%


%% Clear Memory and start timer
clear
clc 

warning('off','MATLAB:nargchk:deprecated')

%% User-specified parameters
budget_length    = 7;       % Number of budgets
number_classes   = 3;       % Number of aggregate commodity bundles
polynomial_degree= 3;       % Series degree estimator
estimator        = 0;       % 0 = Kernel; 1 = Exogenous Series; 2 = Endogenous Series
genAX            = 1;       % Generate pseudo-agent matrix A and patches X


%% Global parameters
global classes budget_l budget_n start_year end_year periods 
global flag_genAX 


%% Flags
% flag_genAX = 1 iff we want to generate matrix A and X and there does not
% exist a pre-generated AX file.
flag_genAX = genAX;
nameAX= strcat('Input/RUM_',num2str(budget_length),'LengthBudget_',num2str(number_classes),'.mat');
if exist(nameAX) ~= 2 & genAX == 0
    flag_genAX = 1;
end

%% Parameters
% Length of sub-budgets to be called (e.g. budget_l = 8 implies budgets 
% span 75-82, 76-83, ect.).  When there are 3 classes use budget_l <9,
% otherwise it takes a long time to run.
budget_l    = budget_length;     

% The number of commodity bundles is called classes e.g. 3 is food, 
% services, durables).  Model currently allows for classes = 3, 4, or 5.
% This can be further disaggregated by modifying RUM_21_budgets.m
classes     = number_classes;      

% Time parameters:
start_year  = 75;                        % First year for budgets
end_year    = 99;                        % Last year for budgets
periods     = end_year - start_year + 1; % Total number of periods
budget_n    = periods - (budget_l-1);    % Number of sub-budgets


%% Create budgets 
% RUM_21_budgets cleans the dataset and outputs normalized budgets, as well
% as expenditure shares and income for each surveyed household. 
[budgets_all,shares_all,income_all,instrument_all,medexp_all] = RUM_21_budgets_abi;


%% Demand predictions at each budget unconstrained 
[pp, q_, income_long_, shares_long_, for_weights, period_long_, x0, ...
            Nyear_long_] = MC_unconstrained_longbudget(budgets_all,shares_all,income_all, ...
                            medexp_all);
T = size(budgets_all, 1);                        
garp_results = NaN(T-1,1);
for i = 2 : T
    garp_results(i-1) = garp(pp(:,1:i), q_(:,1:i));
end

%% MC Results 
results = NaN(T,7);
for i = 2 : T
    disp(i)
    N_ = sum(Nyear_long_(1:i));
    fw = for_weights(1:N_,1:i,:);
    tic
    [A, b, C, d, ctype, Aeq, beq, lb, ub] = mutcon_constraints_extrans(fw, ...
                                                pp(:,1:i), x0(1:i), ...
                                                Nyear_long_(1:i));
    [xhat, resnorm] = ...
      cplexlsqmilp (C, d, A, b, ...
                    Aeq, beq, [ ], [ ], [ ], lb, [], ctype, [ ], []);
    time = toc;
    N = size(fw, 1);
    qhat_ = NaN(number_classes,i);
    for t = 1 : i
        qhat_(1,t) = sum(fw(:, t, 1).*xhat(1:N_));
        qhat_(2,t) = sum(fw(:, t, 2).*xhat(1:N_));
    end
    qhat_(3,:) = x0(1:i) - sum(pp(1:2,1:i).*qhat_(1:2,:));
    results(i,1:5) = [size(A, 1) size(A, 2) time resnorm N];
    results(i,6) = garp(pp(:,1:i), q_(:,1:i));
    results(i,7) = garp(pp(:,1:i), qhat_);
end



