function [result,M,fail] = garp (pin, qin)
%--------------------------------------------------------------------------
% PURPOSE: garp (p, q) tests the Generalized Axiom of Revealed Preference
% (GARP) and returns 0 for GARP fails or 1 for GARP passes.
%--------------------------------------------------------------------------
% USAGE: result = garp (p, q)
% where: p = price matrix
%        q = quantity matrix
%--------------------------------------------------------------------------
% RETURNS: result = 0 or 1
%--------------------------------------------------------------------------
% REFERENCE: Varian, H. R. (1982): "The Nonparametric Approach to Demand
%            Analysis," *Econometrica*, 50, 945-973.
%--------------------------------------------------------------------------

    %----------------------------------------------------------------------
    % CHECK USER ERRORS
    %----------------------------------------------------------------------

    % check number of arguments
    if nargin ~= 2;
        error ('Not enough input arguments.');
    end;

    % check price/quantity consistency
    if sum (size (qin) == size (pin), 2) ~= 2;
        error ('Price and quantity dimensions must agree.');
    end;

    % check minimum number of goods
    if size (qin, 1) < 2;
        error ('Not enough goods.');
    end;


    %----------------------------------------------------------------------
    % BEGIN MAIN PROGRAM
    %----------------------------------------------------------------------

    % initialize revealed preference matrix
    M0 = eye (size (qin, 2));

    % summarize directly revealed preferences
    for i = 1 : size (qin, 2);
        for j = 1 : size (qin, 2);
            M0(i, j) = (pin(:, i)' * qin(:, i) >= pin(:, i)' * qin(:, j));
        end;
    end;

    % summarize revealed preferences
    Mk = M0;
    for k = 1 : size (qin, 2);
        Mk1 = Mk;
        for i = 1 : size (qin, 2);
            for j = 1 : size (qin, 2);
                Mk(i, j) = max ([Mk1(i, j); Mk1(i, k) * Mk1(k, j)]);
            end;
        end;
    end
    M = Mk; 
    
    % test GARP
    result = 1;
    fail = zeros( size(qin, 2));
    for i = 1 : size (qin, 2);
        for j = 1 : size (qin, 2);
            if M(i, j) == 1;
                if pin(:, j)' * qin(:, j) > pin(:, j)' * qin(:, i);
                    result = 0;
                    fail(i,j) = 1;
                end;
            end;
        end;
    end;

    %----------------------------------------------------------------------
    % END MAIN PROGRAM
    %----------------------------------------------------------------------